@@ -14,7 +14,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
  * License for more details.
  *
- * RCSID $Id: ipsec_kversion.h,v 1.15.2.2 2005/09/01 01:57:19 paul Exp $
+ * RCSID $Id: ipsec_kversion.h,v 1.15.2.7 2006/01/11 02:02:59 mcr Exp $
  */
 #define	_FREESWAN_KVERSIONS_H	/* seen it, no need to see it again */
 
@@ -123,11 +123,22 @@
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,13)
 #define HAVE_SOCK_SECURITY
-
 /* skb->nf_debug disappared completely in 2.6.13 */
 #define HAVE_SKB_NF_DEBUG
 #endif
 
+#define SYSCTL_IPSEC_DEFAULT_TTL sysctl_ip_default_ttl                      
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
+/* skb->stamp changed to skb->tstamp in 2.6.14 */
+#define HAVE_TSTAMP
+#define HAVE_INET_SK_SPORT
+#undef  SYSCTL_IPSEC_DEFAULT_TTL
+#define SYSCTL_IPSEC_DEFAULT_TTL IPSEC_DEFAULT_TTL
+#else
+#define HAVE_SKB_LIST
+#endif
+
+
 #ifdef NET_21
 #  include <linux/in6.h>
 #else
@@ -232,6 +243,22 @@
 
 /*
  * $Log: ipsec_kversion.h,v $
+ * Revision 1.15.2.7  2006/01/11 02:02:59  mcr
+ * updated patches and DEFAULT_TTL code to work
+ *
+ * Revision 1.15.2.6  2006/01/03 19:25:02  ken
+ * Remove duplicated #ifdef for TTL fix - bad patch
+ *
+ * Revision 1.15.2.5  2006/01/03 18:06:33  ken
+ * Fix for missing sysctl default ttl
+ *
+ * Revision 1.15.2.4  2005/11/27 21:40:14  paul
+ * Pull down TTL fixes from head. this fixes "Unknown symbol sysctl_ip_default_ttl"
+ * in for klips as module.
+ *
+ * Revision 1.15.2.3  2005/11/22 04:11:52  ken
+ * Backport fixes for 2.6.14 kernels from HEAD
+ *
  * Revision 1.15.2.2  2005/09/01 01:57:19  paul
  * michael's fixes for 2.6.13 from head
  *
