@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: dpd.c,v 1.13.2.1 2005/01/01 00:05:25 ken Exp $
+ * RCSID $Id: dpd.c,v 1.29.2.6 2005/12/27 22:30:36 ken Exp $
  */
 
 #include <stdio.h>
@@ -122,30 +122,38 @@
  *
  */ 
 
-void
+stf_status
 dpd_init(struct state *st)
 {
     /**
-    * Used to store the 1st state 
-    */
+     * Used to store the 1st state 
+     */
     struct state *p1st;
 
-    DBG_log("attempting to enable DPD");
-
     /* find the related Phase 1 state */
     p1st = find_state(st->st_icookie, st->st_rcookie,
-            &st->st_connection->spd.that.host_addr, 0);
+		      &st->st_connection->spd.that.host_addr, 0);
 
     if (p1st == NULL) {
-        loglog(RC_LOG_SERIOUS, "could not find phase 1 state for DPD");
-	passert(0);  /* we need to do something about this, but what? */
-	return;
+        loglog(RC_LOG_SERIOUS, "DPD: Serious: could not find phase 1 state for DPD");
+
+	/*
+	 * if the phase 1 state has gone away, it really should have
+	 * deleted all of its children.
+	 * Why would this happen? because a quick mode SA can take
+	 * some time to create (DNS lookups for instance), and the phase 1
+	 * might have been taken down for some reason in the meantime.
+	 * We really can not do anything here --- attempting to invoke
+	 * the DPD action would be a good idea, but we really should
+	 * do that outside this function.
+	 */
+	return STF_FAIL;
     }
 
     /* if it was enabled, and we haven't turned it on already */
     if (p1st->hidden_variables.st_dpd) {
 	time_t n = now();
-	openswan_log("Dead Peer Detection (RFC 3706) enabled");
+	openswan_log("Dead Peer Detection (RFC 3706): enabled");
 
 	if(st->st_dpd_event == NULL
 	   || (st->st_connection->dpd_delay + n) < st->st_dpd_event->ev_time) {
@@ -154,7 +162,7 @@ dpd_init(struct state *st)
 	}
 
     } else {
-	DBG_log("DPD not enabled because peer did not advertise it");
+      openswan_log("Dead Peer Detection (RFC 3706): not enabled because peer did not advertise it");
     }
 
     if(p1st != st) {
@@ -164,6 +172,7 @@ dpd_init(struct state *st)
 	    delete_dpd_event(p1st);
 	}
     }
+    return STF_OK;
 }
 
 bool was_eroute_idle(struct state *st, time_t since_when);
@@ -179,7 +188,7 @@ dpd_sched_timeout(struct state *p1st, time_t tm, time_t timeout)
     if (p1st->st_dpd_event == NULL
 	|| p1st->st_dpd_event->ev_time > tm + timeout)
     {
-	DBG(DBG_DPD, DBG_log("scheduling timeout to %lu"
+	DBG(DBG_DPD, DBG_log("DPD: Debug: scheduling timeout to %lu"
 			     , timeout));
         delete_dpd_event(p1st);
         event_schedule(EVENT_DPD_TIMEOUT, timeout, p1st);
@@ -202,8 +211,9 @@ dpd_outI(struct state *p1st, struct state *st, bool eroute_care
     bool   eroute_idle;
     time_t nextdelay;
 
-    DBG(DBG_DPD, DBG_log("processing dpd for state #%lu"
-			 , st->st_serialno));
+    DBG(DBG_DPD, DBG_log("DPD: Debug: processing dpd for state #%lu (\"%s\")"
+			 , st->st_serialno
+			 , st->st_connection->name));
 
     /* If no DPD, then get out of here */
     if (!st->hidden_variables.st_dpd)
@@ -235,7 +245,7 @@ dpd_outI(struct state *p1st, struct state *st, bool eroute_care
     /* has there been enough activity of late? */
     if(nextdelay > 0) {
 	/* Yes, just reschedule "phase 2" */
-	DBG(DBG_DPD, DBG_log("not yet time for dpd event: %lu < %lu"
+	DBG(DBG_DPD, DBG_log("DPD: Debug: not yet time for dpd event: %lu < %lu"
 			     , tm, (p1st->st_last_dpd + delay)));
 	event_schedule(EVENT_DPD, nextdelay, st);
 	return;
@@ -250,11 +260,11 @@ dpd_outI(struct state *p1st, struct state *st, bool eroute_care
      * check the phase 2, if we are supposed to,
      * and return if it is active recently 
      */
-    if(eroute_care && !st->nat_traversal) {
+    if(eroute_care && !st->hidden_variables.st_nat_traversal) {
       
 	eroute_idle = was_eroute_idle(st, delay);
 	if(!eroute_idle) {
-	    DBG(DBG_DPD, DBG_log("dpd out event not sent, phase 2 active"));
+	    DBG(DBG_DPD, DBG_log("DPD: Debug: out event not sent, phase 2 active"));
 	    
 	    /* update phase 2 time stamp only */
 	    st->st_last_dpd = tm;
@@ -288,11 +298,16 @@ dpd_outI(struct state *p1st, struct state *st, bool eroute_care
      */
     dpd_sched_timeout(p1st, tm, timeout);
 
-    DBG(DBG_DPD, DBG_log("sending R_U_THERE %u", seqno));
+    DBG(DBG_DPD, DBG_log("DPD: Debug: sending R_U_THERE %u to %s:%d (state #%lu)"
+			 , seqno
+			 , ip_str(&p1st->st_remoteaddr)
+			 , p1st->st_remoteport
+			 , p1st->st_serialno));
+
     if (send_isakmp_notification(p1st, R_U_THERE
 				 , &seqno, sizeof(seqno)) != STF_IGNORE)
     {   
-        loglog(RC_LOG_SERIOUS, "DPD Error: could not send R_U_THERE");
+        loglog(RC_LOG_SERIOUS, "DPD: Serious: could not send R_U_THERE");
         return;
     }
         
@@ -323,7 +338,7 @@ p2_dpd_outI1(struct state *p2st)
 
     if (st == NULL)
     {
-        loglog(RC_LOG_SERIOUS, "DPD Error: could not find newest phase 1 state");
+        loglog(RC_LOG_SERIOUS, "DPD: Serious: could not find newest phase 1 state");
         return;
     }
 
@@ -361,12 +376,12 @@ dpd_inI_outR(struct state *p1st
         
     if (!IS_ISAKMP_SA_ESTABLISHED(p1st->st_state))
     {   
-        loglog(RC_LOG_SERIOUS, "DPD Error: received R_U_THERE for unestablished ISKAMP SA");
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: received R_U_THERE for unestablished ISKAMP SA");
         return STF_IGNORE;
     }
     if (n->isan_spisize != COOKIE_SIZE * 2 || pbs_left(pbs) < COOKIE_SIZE * 2)
     {
-        loglog(RC_LOG_SERIOUS, "DPD Error: R_U_THERE has invalid SPI length (%d)", n->isan_spisize);
+        loglog(RC_LOG_SERIOUS, "DPD: Error: R_U_THERE has invalid SPI length (%d)", n->isan_spisize);
         return STF_FAIL + PAYLOAD_MALFORMED;
     }
         
@@ -374,37 +389,36 @@ dpd_inI_outR(struct state *p1st
     {
         /* RFC states we *SHOULD* check cookies, not MUST.  So invalid
            cookies are technically valid, as per Geoffrey Huang */
-        loglog(RC_LOG_SERIOUS, "DPD Error: R_U_THERE has invalid icookie (broken Cisco?)");
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: R_U_THERE has invalid icookie");
     }
     pbs->cur += COOKIE_SIZE;
     
     if (memcmp(pbs->cur, p1st->st_rcookie, COOKIE_SIZE) != 0)
     {
-        loglog(RC_LOG_SERIOUS, "DPD Error: R_U_THERE has invalid rcookie (broken Cisco?)");      
-	return STF_FAIL + INVALID_COOKIE;
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: R_U_THERE has invalid rcookie");      
     }
     pbs->cur += COOKIE_SIZE;
 
     if (pbs_left(pbs) != sizeof(seqno))
     {
-        loglog(RC_LOG_SERIOUS, "DPD Error: R_U_THERE has invalid data length (%d)", (int) pbs_left(pbs));
+        loglog(RC_LOG_SERIOUS, "DPD: Error: R_U_THERE has invalid data length (%d)", (int) pbs_left(pbs));
         return STF_FAIL + PAYLOAD_MALFORMED;
     }
 
     seqno = ntohl(*(u_int32_t *)pbs->cur);
     if (p1st->st_dpd_peerseqno && seqno <= p1st->st_dpd_peerseqno) {
-        loglog(RC_LOG_SERIOUS, "DPD Info: received old or duplicate R_U_THERE");
+        loglog(RC_LOG_SERIOUS, "DPD Warning: received old or duplicate R_U_THERE");
         return STF_IGNORE;
     }
      
-    DBG(DBG_DPD, DBG_log("received R_U_THERE seq:%u time:%lu", seqno, tm));
+    DBG(DBG_DPD, DBG_log("DPD: Debug: received R_U_THERE seq:%u time:%lu (state=#%lu name=\"%s\")" , seqno, tm, p1st->st_serialno, p1st->st_connection->name));
 
     p1st->st_dpd_peerseqno = seqno;
 
     if (send_isakmp_notification(p1st, R_U_THERE_ACK
 				 , pbs->cur, pbs_left(pbs)) != STF_IGNORE)
     {
-        loglog(RC_LOG_SERIOUS, "DPD Info: could not send R_U_THERE_ACK"); 
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: could not send R_U_THERE_ACK"); 
         return STF_IGNORE;
     }
 
@@ -441,13 +455,13 @@ dpd_inR(struct state *p1st
      
     if (!IS_ISAKMP_SA_ESTABLISHED(p1st->st_state))
     {
-        loglog(RC_LOG_SERIOUS, "recevied R_U_THERE_ACK for unestablished ISKAMP SA");
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: recieved R_U_THERE_ACK for unestablished ISKAMP SA");
         return STF_FAIL;
     }
 
    if (n->isan_spisize != COOKIE_SIZE * 2 || pbs_left(pbs) < COOKIE_SIZE * 2)
     {
-        loglog(RC_LOG_SERIOUS, "R_U_THERE_ACK has invalid SPI length (%d)", n->isan_spisize);
+        loglog(RC_LOG_SERIOUS, "DPD: Error: R_U_THERE_ACK has invalid SPI length (%d)", n->isan_spisize);
         return STF_FAIL + PAYLOAD_MALFORMED;
     }
      
@@ -455,7 +469,7 @@ dpd_inR(struct state *p1st
     {
         /* RFC states we *SHOULD* check cookies, not MUST.  So invalid
            cookies are technically valid, as per Geoffrey Huang */
-        loglog(RC_LOG_SERIOUS, "R_U_THERE_ACK has invalid icookie");
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: R_U_THERE_ACK has invalid icookie");
     }
     pbs->cur += COOKIE_SIZE;
     
@@ -463,26 +477,28 @@ dpd_inR(struct state *p1st
     {
         /* RFC states we *SHOULD* check cookies, not MUST.  So invalid
            cookies are technically valid, as per Geoffrey Huang */
-        loglog(RC_LOG_SERIOUS, "R_U_THERE_ACK has invalid rcookie");
+        loglog(RC_LOG_SERIOUS, "DPD: Warning: R_U_THERE_ACK has invalid rcookie ");
     }
     pbs->cur += COOKIE_SIZE;
     
     if (pbs_left(pbs) != sizeof(seqno))
     {
-        loglog(RC_LOG_SERIOUS, "R_U_THERE_ACK has invalid data length (%d)", (int) pbs_left(pbs));
+        loglog(RC_LOG_SERIOUS, "DPD: Error: R_U_THERE_ACK has invalid data length (%d)", (int) pbs_left(pbs));
         return STF_FAIL + PAYLOAD_MALFORMED;
     }
         
     seqno = ntohl(*(u_int32_t *)pbs->cur);
-    DBG(DBG_DPD, DBG_log("R_U_THERE_ACK, seqno received: %u expected: %u",
-			 seqno, p1st->st_dpd_expectseqno));
+    DBG(DBG_DPD, DBG_log("DPD: Debug: R_U_THERE_ACK, seqno received: %u expected: %u (state=#%lu)",
+			 seqno, p1st->st_dpd_expectseqno, p1st->st_serialno));
+
     if (!p1st->st_dpd_expectseqno && seqno != p1st->st_dpd_expectseqno) {
-        loglog(RC_LOG_SERIOUS, "R_U_THERE_ACK has unexpected sequence number");
-        return STF_FAIL + PAYLOAD_MALFORMED;
+        loglog(RC_LOG_SERIOUS, "DPD: Error: R_U_THERE_ACK has unexpected sequence number (expected: %u got: %u", seqno, p1st->st_dpd_expectseqno);
+	p1st->st_dpd_expectseqno = 0;
+	/* do not update time stamp, so we'll send a new one sooner */
+    } else {
+	/* update the time stamp */
+	p1st->st_last_dpd = tm;
     }
-     
-    /* update the time stamp */
-    p1st->st_last_dpd = tm;
 
     p1st->st_dpd_expectseqno = 0;
 
@@ -515,37 +531,44 @@ dpd_timeout(struct state *st)
     struct connection *c = st->st_connection;
     action = st->st_connection->dpd_action;
     
-    passert(action == DPD_ACTION_HOLD || action == DPD_ACTION_CLEAR);
+    /* probably wrong thing to assert here */
+    passert(action == DPD_ACTION_HOLD
+	    || action == DPD_ACTION_CLEAR
+	    || action == DPD_ACTION_RESTART);
         
-    loglog(RC_LOG_SERIOUS, "DPD: Info: No response from peer - declaring peer dead");
-
     /** delete the state, which is probably in phase 2 */
     set_cur_connection(c);
-    openswan_log("terminating SAs using this connection");
-    delete_states_by_connection(c, TRUE);  
-    reset_cur_connection();
+
+    openswan_log("DPD: Info: No response from peer - declaring peer dead");
 
     switch(action) {
     case DPD_ACTION_HOLD:
 	/** dpdaction=hold - Wipe the SA's but %trap the eroute so we don't
 	    leak traffic.  Also, being in %trap means new packets will
 	    force an initiation of the conn again.  */
-	loglog(RC_LOG_SERIOUS, "DPD: Info: Putting connection into %%trap");
+	openswan_log("DPD: Info: Putting connection into %%trap");
+	delete_states_by_connection(c, TRUE);  
 	break;
 
     case DPD_ACTION_CLEAR:
         /** dpdaction=clear - Wipe the SA & eroute - everything */
     
-        loglog(RC_LOG_SERIOUS, "DPD: Info: Clearing Connection");
-        unroute_connection(c);
+        openswan_log("DPD: Info: Clearing Connection");
+	delete_states_by_connection(c, TRUE);
+	DBG(DBG_DPD, DBG_log("DPD: Debug: issuing unroute_connection()"));
+        unroute_connection(c);        /* --unroute */
 	break;
 
     case DPD_ACTION_RESTART:
-	/** dpdactin=restart - immediate renogotiate the connection. */
-        loglog(RC_LOG_SERIOUS, "DPD: Info: restarting Connection");
+	/** dpdaction=restart - immediate renegotiate the connection. */
+        openswan_log("DPD: Info: Restarting Connection");
+
+	/* we replace the SA so that we do it in a rational place */
+	delete_event(st);
 	event_schedule(EVENT_SA_REPLACE, 0, st);
 	break;
     }
+    reset_cur_connection();
 }
 
 
