@@ -15,7 +15,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ikev1_quick.c,v 1.3 2005/05/18 19:29:29 mcr Exp $
+ * RCSID $Id: ikev1_quick.c,v 1.3.2.3 2006/04/04 22:17:49 paul Exp $
  */
 
 #include <stdio.h>
@@ -1533,6 +1533,19 @@ quick_inI1_outR1_authtail(struct verify_oppo_bundle *b
 	struct connection *p = find_client_connection(c
 	    , our_net, his_net, b->my.proto, b->my.port, b->his.proto, b->his.port);
 
+#ifdef NAT_TRAVERSAL
+#ifdef I_KNOW_TRANSPORT_MODE_HAS_SECURITY_CONCERN_BUT_I_WANT_IT
+    if( (p1st->hidden_variables.st_nat_traversal & NAT_T_DETECTED)
+       && !(p1st->st_policy & POLICY_TUNNEL)
+       && (p1st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))
+       && (p == NULL) )
+        {
+          p = c;
+          DBG(DBG_CONTROL, DBG_log("using (something) old for transport mode connection \"%s\"", p->name));
+        }
+#endif
+#endif
+
 	if (p == NULL)
 	{
 	    /* This message occurs in very puzzling circumstances
@@ -1910,14 +1923,6 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
     }
 
 #ifdef NAT_TRAVERSAL
-    if ((st->hidden_variables.st_nat_traversal & NAT_T_WITH_NATOA) &&
-	(st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME)) &&
-	(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT)) {
-	/** Send NAT-OA if our address is NATed and if we use Transport Mode */
-	if (!nat_traversal_add_natoa(ISAKMP_NEXT_NONE, &md->rbody, md->st)) {
-	    return STF_INTERNAL_ERROR;
-	}
-    }
     if ((st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) &&
 	(st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TRANSPORT) &&
 	(c->spd.that.has_client)) {
@@ -1945,7 +1950,10 @@ quick_inI1_outR1_cryptotail(struct qke_continuation *qke
     /* encrypt message, except for fixed part of header */
     
     if (!encrypt_message(&md->rbody, st))
+    {
+	delete_ipsec_sa(st, TRUE);
 	return STF_INTERNAL_ERROR;	/* ??? we may be partly committed */
+    }
 
     DBG(DBG_CONTROLMORE, DBG_log("finished processing quick inI1"));
     return STF_OK;
@@ -2069,7 +2077,10 @@ quick_inR1_outI2(struct msg_digest *md)
     /* encrypt message, except for fixed part of header */
 
     if (!encrypt_message(&md->rbody, st))
+    {
+	delete_ipsec_sa(st, FALSE);
 	return STF_INTERNAL_ERROR;	/* ??? we may be partly committed */
+    }
 
     {
       DBG(DBG_CONTROLMORE, DBG_log("inR1_outI2: instance %s[%ld], setting newest_ipsec_sa to #%ld (was #%ld) (spd.eroute=#%ld)"
@@ -2090,6 +2101,7 @@ quick_inR1_outI2(struct msg_digest *md)
 	on this conn, so initialize it */
     if (st->st_connection->dpd_delay && st->st_connection->dpd_timeout) {
 	if(dpd_init(st) != STF_OK) {
+	    delete_ipsec_sa(st, FALSE);
 	    return STF_FAIL;
 	}
     }
@@ -2145,6 +2157,7 @@ quick_inI2(struct msg_digest *md)
 	on this conn, so initialize it */
     if(st->st_connection->dpd_delay && st->st_connection->dpd_timeout) {
 	if(dpd_init(st) != STF_OK) {
+	    delete_ipsec_sa(st, FALSE);
 	    return STF_FAIL;
 	}
     }
