@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: server.c,v 1.103 2004/11/30 02:28:07 mcr Exp $
+ * RCSID $Id: server.c,v 1.109.2.2 2005/11/29 03:02:16 paul Exp $
  */
 
 #include <stdio.h>
@@ -42,6 +42,7 @@
 #include <sys/queue.h>
 #include <sys/resource.h>
 #include <sys/wait.h>
+#include <sys/queue.h>
 
 #include <openswan.h>
 
@@ -70,6 +71,7 @@
 #include "dnskey.h"	/* needs keys.h and adns.h */
 #include "whack.h"	/* for RC_LOG_SERIOUS */
 #include "pluto_crypt.h" /* cryptographic helper functions */
+#include "udpfromto.h"
 
 #include <pfkeyv2.h>
 #include <pfkey.h>
@@ -87,6 +89,9 @@ static const int on = TRUE;	/* by-reference parameter; constant, we hope */
 
 bool no_retransmits = FALSE;
 
+/* list of interface devices */
+LIST_HEAD(,iface_dev) interface_dev;
+
 /* control (whack) socket */
 int ctl_fd = NULL_FD;	/* file descriptor of control (whack) socket */
 struct sockaddr_un ctl_addr = { AF_UNIX, DEFAULT_CTLBASE CTL_SUFFIX };
@@ -105,6 +110,8 @@ init_ctl_socket(void)
 {
     err_t failed = NULL;
 
+    LIST_INIT(&interface_dev);
+
     delete_ctl_socket();	/* preventative medicine */
     ctl_fd = socket(AF_UNIX, SOCK_STREAM, 0);
     if (ctl_fd == -1)
@@ -116,7 +123,11 @@ init_ctl_socket(void)
     else
     {
 	/* to keep control socket secure, use umask */
+#ifdef PLUTO_GROUP_CTL
+	mode_t ou = umask(~(S_IRWXU|S_IRWXG));
+#else
 	mode_t ou = umask(~S_IRWXU);
+#endif
 
 	if (bind(ctl_fd, (struct sockaddr *)&ctl_addr
 	, offsetof(struct sockaddr_un, sun_path) + strlen(ctl_addr.sun_path)) < 0)
@@ -124,6 +135,20 @@ init_ctl_socket(void)
 	umask(ou);
     }
 
+#ifdef PLUTO_GROUP_CTL
+    {
+	struct group *g;
+
+	g = getgrnam("pluto");
+	if(g != NULL) {
+	    if(fchown(ctl_fd, -1, g->gr_gid) != 0) {
+		loglog(RC_LOG_SERIOUS, "Can not chgrp ctl fd(%d) to gid=%d: %s\n"
+		       , ctl_fd, g->gr_gid, strerror(errno));
+	    }
+	}
+    }
+#endif
+
     /* 5 is a haphazardly chosen limit for the backlog.
      * Rumour has it that this is the max on BSD systems.
      */
@@ -187,23 +212,36 @@ delete_info_socket(void)
 
 bool listening = FALSE;	/* should we pay attention to IKE messages? */
 
-struct iface *interfaces = NULL;	/* public interfaces */
+struct iface_port  *interfaces = NULL;	/* public interfaces */
 
 /* Initialize the interface sockets. */
 
 static void
 mark_ifaces_dead(void)
 {
-    struct iface *p;
+    struct iface_port *p;
 
     for (p = interfaces; p != NULL; p = p->next)
 	p->change = IFN_DELETE;
 }
 
+static void
+free_dead_iface_dev(struct iface_dev *id)
+{
+    if(--id->id_count == 0) {
+	pfree(id->id_vname);
+	pfree(id->id_rname);
+
+	LIST_REMOVE(id, id_entry);
+
+	pfree(id);
+    }
+}
+
 static void
 free_dead_ifaces(void)
 {
-    struct iface *p;
+    struct iface_port *p;
     bool some_dead = FALSE
 	, some_new = FALSE;
 
@@ -211,8 +249,10 @@ free_dead_ifaces(void)
     {
 	if (p->change == IFN_DELETE)
 	{
-	    openswan_log("shutting down interface %s/%s %s"
-		, p->vname, p->rname, ip_str(&p->addr));
+	    openswan_log("shutting down interface %s/%s %s:%d"
+			 , p->ip_dev->id_vname
+			 , p->ip_dev->id_rname
+			 , ip_str(&p->ip_addr), p->port);
 	    some_dead = TRUE;
 	}
 	else if (p->change == IFN_ADD)
@@ -223,18 +263,22 @@ free_dead_ifaces(void)
 
     if (some_dead)
     {
-	struct iface **pp;
+	struct iface_port **pp;
 
 	release_dead_interfaces();
 	for (pp = &interfaces; (p = *pp) != NULL; )
 	{
 	    if (p->change == IFN_DELETE)
 	    {
+		struct iface_dev *id;
+
 		*pp = p->next;	/* advance *pp */
-		pfree(p->vname);
-		pfree(p->rname);
 		close(p->fd);
+
+		id = p->ip_dev;
 		pfree(p);
+
+		free_dead_iface_dev(id);
 	    }
 	    else
 	    {
@@ -293,7 +337,7 @@ use_interface(const char *rifn)
 #endif
 
 static struct raw_iface *
-find_raw_ifaces4(void)
+find_raw_ifaces(void)
 {
     int j;	/* index into buf */
     struct ifconf ifconf;
@@ -304,11 +348,11 @@ find_raw_ifaces4(void)
     /* get list of interfaces with assigned IPv4 addresses from system */
 
     if (master_sock == -1)
-	exit_log_errno((e, "socket() failed in find_raw_ifaces4()"));
+	exit_log_errno((e, "socket() failed in find_raw_ifaces()"));
 
     if (setsockopt(master_sock, SOL_SOCKET, SO_REUSEADDR
     , (const void *)&on, sizeof(on)) < 0)
-	exit_log_errno((e, "setsockopt() in find_raw_ifaces4()"));
+	exit_log_errno((e, "setsockopt() in find_raw_ifaces()"));
 
     /* bind the socket */
     {
@@ -317,7 +361,7 @@ find_raw_ifaces4(void)
 	happy(anyaddr(AF_INET, &any));
 	setportof(htons(pluto_port), &any);
 	if (bind(master_sock, sockaddrof(&any), sockaddrlenof(&any)) < 0)
-	    exit_log_errno((e, "bind() failed in find_raw_ifaces4()"));
+	    exit_log_errno((e, "bind() failed in find_raw_ifaces()"));
     }
 
     /* Get local interfaces.  See netdevice(7). */
@@ -326,7 +370,7 @@ find_raw_ifaces4(void)
     zero(buf);
 
     if (ioctl(master_sock, SIOCGIFCONF, &ifconf) == -1)
-	exit_log_errno((e, "ioctl(SIOCGIFCONF) in find_raw_ifaces4()"));
+	exit_log_errno((e, "ioctl(SIOCGIFCONF) in find_raw_ifaces()"));
 
     /* Add an entry to rifaces for each interesting interface. */
     for (j = 0; (j+1) * sizeof(*buf) <= (size_t)ifconf.ifc_len; j++)
@@ -360,10 +404,12 @@ find_raw_ifaces4(void)
 	memcpy(auxinfo.ifr_name, buf[j].ifr_name, IFNAMSIZ);
 	if (ioctl(master_sock, SIOCGIFFLAGS, &auxinfo) == -1)
 	    exit_log_errno((e
-		, "ioctl(SIOCGIFFLAGS) for %s in find_raw_ifaces4()"
+		, "ioctl(SIOCGIFFLAGS) for %s in find_raw_ifaces()"
 		, ri.name));
 	if (!(auxinfo.ifr_flags & IFF_UP))
 	    continue;	/* ignore an interface that isn't UP */
+        if (auxinfo.ifr_flags & IFF_SLAVE)
+            continue;   /* ignore slave interfaces; they share IPs with their master */
 
 	/* ignore unconfigured interfaces */
 	if (rs->sin_addr.s_addr == 0)
@@ -384,7 +430,7 @@ find_raw_ifaces4(void)
 }
 
 static struct raw_iface *
-find_raw_ifaces6(void)
+find_raw_ifaces6_linux(void)
 {
 
     /* Get list of interfaces with IPv6 addresses from system from /proc/net/if_inet6).
@@ -531,7 +577,7 @@ create_socket(struct raw_iface *ifp, const char *v_name, int port)
 #endif
 
 #if defined(linux) && defined(KERNEL26_SUPPORT)
-    if (!no_klips && kernel_ops->type == KERNEL_TYPE_LINUX)
+    if (kern_interface == USE_NETKEY)
     {
 	struct sadb_x_policy policy;
 	int level, opt;
@@ -585,6 +631,10 @@ create_socket(struct raw_iface *ifp, const char *v_name, int port)
 	return -1;
     }
     setportof(htons(pluto_port), &ifp->addr);
+
+    /* we are going to use udpfromto.c, so initialize it */
+    udpfromto_init(fd);
+
     return fd;
 }
 
@@ -639,7 +689,7 @@ process_raw_ifaces(struct raw_iface *rifaces)
 		     * "after" allows us to avoid double reporting.
 		     */
 #if defined(linux) && defined(KERNEL26_SUPPORT)
-		    if (!no_klips && kernel_ops->type == KERNEL_TYPE_LINUX)
+		    if (kern_interface == USE_NETKEY)
 		    {
 			if (after)
 			{
@@ -664,7 +714,7 @@ process_raw_ifaces(struct raw_iface *rifaces)
 	    continue;
 
 #if defined(linux) && defined(KERNEL26_SUPPORT)
-	if (!no_klips && kernel_ops->type == KERNEL_TYPE_LINUX)
+	if (kern_interface == USE_NETKEY)
 	{
 	    v = ifp;
 	    goto add_entry;
@@ -674,7 +724,7 @@ process_raw_ifaces(struct raw_iface *rifaces)
 	/* what if we didn't find a virtual interface? */
 	if (v == NULL)
 	{
-	    if (no_klips)
+	    if (kern_interface == NO_KERNEL)
 	    {
 		/* kludge for testing: invent a virtual device */
 		static const char fvp[] = "virtual";
@@ -701,11 +751,12 @@ process_raw_ifaces(struct raw_iface *rifaces)
 add_entry:
 #endif
 	{
-	    struct iface **p = &interfaces;
+	    struct iface_port **p = &interfaces;
 
 	    for (;;)
 	    {
-		struct iface *q = *p;
+		struct iface_port *q = *p;
+		struct iface_dev *id = NULL;
 
 		/* search is over if at end of list */
 		if (q == NULL)
@@ -723,53 +774,79 @@ add_entry:
 		    }
 #endif
 
-		    q = alloc_thing(struct iface, "struct iface");
-		    q->rname = clone_str(ifp->name, "real device name");
-		    q->vname = clone_str(v->name, "virtual device name");
-		    q->addr = ifp->addr;
+		    q = alloc_thing(struct iface_port, "struct iface_port");
+		    id = alloc_thing(struct iface_dev, "struct iface_dev");
+
+		    LIST_INSERT_HEAD(&interface_dev, id, id_entry)
+
+		    q->ip_dev = id;
+		    id->id_rname = clone_str(ifp->name, "real device name");
+		    id->id_vname = clone_str(v->name, "virtual device name");
+		    id->id_count++;
+
+		    q->ip_addr = ifp->addr;
 		    q->fd = fd;
 		    q->next = interfaces;
 		    q->change = IFN_ADD;
+		    q->port = pluto_port;
+		    q->ike_float = FALSE;
+
 		    interfaces = q;
-		    openswan_log("adding interface %s/%s %s"
-			, q->vname, q->rname, ip_str(&q->addr));
+
+		    openswan_log("adding interface %s/%s %s:%d"
+				 , q->ip_dev->id_vname
+				 , q->ip_dev->id_rname
+				 , ip_str(&q->ip_addr)
+				 , q->port);
+
 #ifdef NAT_TRAVERSAL
-		    if (nat_traversal_support_port_floating && addrtypeof(&ifp->addr) == AF_INET) {
+		    /*
+		     * right now, we do not support NAT-T on IPv6, because
+		     * the kernel did not support it, and gave an error
+		     * it one tried to turn it on.
+		     */
+		    if (nat_traversal_support_port_floating
+			&& addrtypeof(&ifp->addr) == AF_INET)
+		    {
 			fd = create_socket(ifp, v->name, NAT_T_IKE_FLOAT_PORT);
 			if (fd < 0) 
 			    break;
 			nat_traversal_espinudp_socket(fd, "IPv4"
 						      , ESPINUDP_WITH_NON_ESP);
-			q = alloc_thing(struct iface, "struct iface");
-			q->rname = clone_str(ifp->name, "real device name");
-			q->vname = clone_str(v->name, "virtual device name");
-			q->addr = ifp->addr;
-			setportof(htons(NAT_T_IKE_FLOAT_PORT), &q->addr);
+			q = alloc_thing(struct iface_port, "struct iface_port");
+			q->ip_dev = id;
+			id->id_count++;
+			
+			q->ip_addr = ifp->addr;
+			setportof(htons(NAT_T_IKE_FLOAT_PORT), &q->ip_addr);
+			q->port = NAT_T_IKE_FLOAT_PORT;
 			q->fd = fd;
 			q->next = interfaces;
 			q->change = IFN_ADD;
 			q->ike_float = TRUE;
 			interfaces = q;
-			openswan_log("adding interface %s/%s %s:%d",
-			q->vname, q->rname, ip_str(&q->addr), NAT_T_IKE_FLOAT_PORT);
+			openswan_log("adding interface %s/%s %s:%d"
+				     , q->ip_dev->id_vname, q->ip_dev->id_rname
+				     , ip_str(&q->ip_addr)
+				     , q->port);
 		    }
 #endif
 		    break;
 		}
 
 		/* search over if matching old entry found */
-		if (streq(q->rname, ifp->name)
-		&& streq(q->vname, v->name)
-		&& sameaddr(&q->addr, &ifp->addr))
+		if (streq(q->ip_dev->id_rname, ifp->name)
+		    && streq(q->ip_dev->id_vname, v->name)
+		    && sameaddr(&q->ip_addr, &ifp->addr))
 		{
 		    /* matches -- rejuvinate old entry */
 		    q->change = IFN_KEEP;
 #ifdef NAT_TRAVERSAL
 		    /* look for other interfaces to keep (due to NAT-T) */
 		    for (q = q->next ; q ; q = q->next) {
-			if (streq(q->rname, ifp->name)
-			    && streq(q->vname, v->name)
-			    && sameaddr(&q->addr, &ifp->addr)) {
+			if (streq(q->ip_dev->id_rname, ifp->name)
+			    && streq(q->ip_dev->id_vname, v->name)
+			    && sameaddr(&q->ip_addr, &ifp->addr)) {
 				q->change = IFN_KEEP;
 			}
 		    }
@@ -797,9 +874,14 @@ void
 find_ifaces(void)
 {
     mark_ifaces_dead();
-    process_raw_ifaces(find_raw_ifaces4());
-    process_raw_ifaces(find_raw_ifaces6());
-
+/* 
+    find_raw_ifaces uses SIOCGIFCONF, which returns ipv4 and ipv6,
+    except on Linux.
+*/
+    process_raw_ifaces(find_raw_ifaces());
+#if defined(linux) 
+    process_raw_ifaces(find_raw_ifaces6_linux());
+#endif
     free_dead_ifaces();	    /* ditch remaining old entries */
 
     if (interfaces == NULL)
@@ -809,11 +891,11 @@ find_ifaces(void)
 void
 show_ifaces_status(void)
 {
-    struct iface *p;
+    struct iface_port *p;
 
     for (p = interfaces; p != NULL; p = p->next)
 	whack_log(RC_COMMENT, "interface %s/%s %s"
-	    , p->vname, p->rname, ip_str(&p->addr));
+	    , p->ip_dev->id_vname, p->ip_dev->id_rname, ip_str(&p->ip_addr));
 }
 
 void
@@ -858,13 +940,20 @@ reapchildren(void)
     struct rusage r;
 
     sigchildflag = FALSE;
-    while((child = wait3(&status, WNOHANG, &r)) != 0) {
+    errno=0;
+
+    while((child = wait3(&status, WNOHANG, &r)) > 0) {
 	/* got a child to reap */
 	if(adns_reapchild(child, status)) continue;
 	if(pluto_crypt_handle_dead_child(child, status)) continue;
 	
 	openswan_log("child pid=%d (status=%d) is not my child!", child, status);
     }
+    
+    if(child == -1) {
+	openswan_log("reapchild failed with errno=%d %s",
+		     errno, strerror(errno));
+    }
 }
 
 /* call_server listens for incoming ISAKMP packets and Whack messages,
@@ -873,7 +962,7 @@ reapchildren(void)
 void
 call_server(void)
 {
-    struct iface *ifp;
+    struct iface_port *ifp;
 
     /* catch SIGHUP and SIGTERM */
     {
@@ -952,7 +1041,7 @@ call_server(void)
 	    }
 
 #ifdef KLIPS
-	    if (!no_klips)
+	    if (kern_interface != NO_KERNEL)
 	    {
 		int fd = *kernel_ops->async_fdp;
 
@@ -1052,7 +1141,8 @@ call_server(void)
 	    }
 
 #ifdef KLIPS
-	    if (!no_klips && FD_ISSET(*kernel_ops->async_fdp, &readfds))
+	    if (kern_interface != NO_KERNEL
+		&& FD_ISSET(*kernel_ops->async_fdp, &readfds))
 	    {
 		passert(ndes > 0);
 		DBG(DBG_CONTROL,
